/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Copyright (C) 2005  Luca Bellonda <lbell at tsc4.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 #include <config.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include <glib.h>
#include <gmodule.h>
#include <gtk/gtkmain.h>
#include <gtk/gtkiconfactory.h>
#include <gtk/gtkitemfactory.h>
#include <gtk/gtkstock.h>
#include <gtk/gtkhpaned.h>
#include <gtk/gtkframe.h>
#include <gtk/gtktreeview.h>
#include <gtk/gtkcellrenderertext.h>
#include <gtk/gtktreeselection.h>
#include <gtk/gtktreemodel.h>
#include <gtk/gtkscrolledwindow.h>
#include <gtk/gtkmessagedialog.h>
#include <libgnome/gnome-i18n.h>
#include <bonobo/bonobo-ui-component.h>
#include <bonobo/bonobo-ui-util.h>
#include <bonobo/bonobo-control.h>
#include <libplanner/mrp-task.h>
#include "planner-view.h"
#include "planner-cell-renderer-date.h"
#include "planner-task-dialog.h"
#include "planner-resource-dialog.h"
#include "planner-gantt-model.h"
#include "planner-task-tree.h"
#include "planner-gantt-chart.h"
#include "planner-gantt-print.h"
#include "planner_col_spc.h"

void view_column_def_init( ViewColumnDef *self, const ColTaskEnum uid, const gchar *title, const gboolean editable, const gfloat alignment, const int width )
{
	g_assert(NULL!=self);
	self->uid       = uid; 
	self->title     = g_strdup(title);
	self->editable  = editable ;
	self->alignment = alignment ;
	self->width     = width;
}

ViewColumnDef *view_column_new( const ColTaskEnum uid, const gchar *title, const gboolean editable, const gfloat alignment, const int width )
{
	ViewColumnDef *new_def = (ViewColumnDef*)g_new0(ViewColumnDef,1);
	if(NULL!=new_def)
		view_column_def_init( new_def, uid, title, editable, alignment, width );
	return new_def;
}

void view_column_set_title( ViewColumnDef *self, const gchar *newTitle )
{
//printf("a 0\n" );
	//g_assert(NULL!=self);
	//g_warning(NULL!=newTitle);
//printf("a 1\n" );
	if(NULL!=self->title)
	{
//printf("a 2\n" );	
		g_free(self->title);
		self->title = NULL ;
	}
//printf("a 3\n" );
	if(NULL!=newTitle)
		self->title = g_strdup(newTitle);
	else
		self->title = g_strdup("");
}

gchar* view_column_get_title_transl_dup( ViewColumnDef *self )
{
	if(NULL!=self->title)
	{
		if( self->title[0] == '\n' )
			return g_strdup_printf( "%s", &self->title[1] );
		return g_strdup(self->title);
	} else
		return g_strdup("");
}
/* returns the textual description of the column */
gchar *view_column_get_type_string_const(ViewColumnDef *self)
{
	switch(self->uid)
	{
	default: return "?";
 	case COL_TM_WBS: return "COL_TM_WBS";
	case COL_TM_NAME: return "COL_TM_NAME";
	case COL_TM_START: return "COL_TM_START";
	case COL_TM_FINISH: return "COL_TM_FINISH";
	case COL_TM_DURATION: return "COL_TM_DURATION";
	case COL_TM_WORK: return "COL_TM_WORK";
	case COL_TM_SLACK: return "COL_TM_SLACK";
	case COL_TM_WEIGHT: return "COL_TM_WEIGHT";
	case COL_TM_EDITABLE: return "COL_TM_EDITABLE";
	case COL_TM_TASK: return "COL_TM_TASK";
	case COL_TM_COST: return "COL_TM_COST";
	case COL_TM_RESOURCE: return "COL_TM_RESOURCE";
	case COL_TM_CONSTRAINT: return "COL_TM_CONSTRAINT";
	case COL_TM_NOTE: return "COL_TM_NOTE";
	case COL_TM_PERCENTDONE: return "COL_TM_PERCENTDONE";
	case COL_TM_CARDINE: return "COL_TM_CARDINE";
	case COL_TM_FIXEDSTARTDATE: return "COL_TM_FIXEDSTARTDATE";
	case COL_TM_STATEICON: return "COL_TM_STATEICON";
	}
}
void view_column_end( ViewColumnDef *self )
{
	g_assert(NULL!=self);
	if( NULL!=self)
	{
		g_free(self->title);
		self->title = NULL ; /* to debug */
		g_free(self);
	}
}


/* Build the default tree for the gantt view */
GList *view_column_get_default_gantt()
{
	GList    *col_list = NULL;/* relies on g_abort */
	col_list = g_list_append (col_list, view_column_new(COL_TM_NOTE,_("\nNote"),FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_WBS,_("\nWBS"),FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_NAME,_("\nName"),FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_START,_("\nStart"),FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_WORK,  _("\nWork"), FALSE,TV_ALIGN_LEFT,100));
/*	col_list = g_list_append (col_list, view_column_new(*COL_TM_DURATION, _("\nDuration"),*****s*/
	col_list = g_list_append (col_list, view_column_new(COL_TM_CONSTRAINT, _("\nConstraint"),FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_SLACK,_("\nSlack"),FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_RESOURCE,_("\nRisorse"),FALSE,TV_ALIGN_LEFT,100));
	return col_list ;
}

void view_columns_end(GList *col_list)
{
	GList *l;
	if(NULL==col_list);
		return ;
	for (l = col_list; NULL != l; l = l->next) {
		view_column_end( (ViewColumnDef*) l->data );
	}
	if(NULL!=col_list)
		g_list_free(col_list);
}

GList *view_columns_duplicate(GList *col_list)
{
	GList *l;
	GList    *cloned_list = NULL;
	g_assert(NULL!=col_list);	
	for (l = col_list; NULL != l; l = l->next) {
		ViewColumnDef *to_clone = (ViewColumnDef*) l->data ;
		cloned_list = g_list_append (cloned_list , view_column_new(to_clone->uid, to_clone->title,
					to_clone->editable, to_clone->alignment, to_clone->width ) );
	}
	return cloned_list ;
}

GList *view_column_get_default_task()
{
	GList    *col_list = NULL;
	col_list = g_list_append (col_list, view_column_new(COL_TM_WBS,_("\nWBS"),FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_NAME,_("\nName"),FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_START,_("\nStart"),FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_FINISH,_("\nFinish"),FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_WORK,  _("\nWork"), FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_SLACK,_("\nSlack"),FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_COST,_("\nCost"),FALSE,TV_ALIGN_LEFT,100));
	return col_list ;
}

