/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Copyright (C) 2005 Luca Bellonda lbell-at-tsc4.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <config.h>
#include <time.h>
#include <string.h>
#include <stdlib.h>
#include <libgnome/gnome-i18n.h>
#include <glade/glade.h>
#include <gtk/gtk.h>
#include <libplanner/mrp-object.h>
#include <libplanner/mrp-project.h>
#include "planner-cell-renderer-list.h"
#include "planner-assignment-model.h"
#include "planner-predecessor-model.h"
#include "planner-task-cmd.h"
#include "planner_task_columns_dialog.h"
/*#include "planner_task_columns_model.h"*/

enum {
        EDIT_TASK_COLUMNS_ID,
        EDIT_TASK_COLUMNS_TITLE,
	/*EDIT_TASK_COLUMNS_PTR,*/
        NUMBER_OF_EDIT_TASK_COLUMNS_COLS
};

typedef struct {
	PlannerWindow *main_window;
	GtkWidget     *dialog;
	GtkWidget     *w_shown_columns_list;
	GtkWidget     *w_hidden_columns_list;
	GtkWidget     *w_move_up_button;
	GtkWidget     *w_move_down_button;
	GtkWidget     *w_add_button;
	GtkWidget     *w_remove_button;
	GList         *list_shown_columns;
	GList         *list_columns;
} ECDialogData ;


/* Keep the dialogs here so that we can just raise the dialog if it's
 * opened twice for the same task.
 */
#define DIALOG_GET_DATA(d) g_object_get_data ((GObject*)d, "data")

/* the list of visible columns */
static void
task_columns_dialog_label_edited(GtkCellRendererText *cell,
		       gchar               *path_string,
		       gchar               *new_text,
		       gpointer             data)
{
	ECDialogData        *dialog;
	GtkTreeView         *tree;
	GtkTreeModel        *model;
	GtkTreePath         *path;
	GtkTreeIter          iter;
	ViewColumnDef       *pDef = NULL ;
	gchar               *txt = NULL; 

	dialog = (ECDialogData*)data;
	tree = GTK_TREE_VIEW(dialog->w_shown_columns_list);
	model = gtk_tree_view_get_model (tree);

	path = gtk_tree_path_new_from_string (path_string);
	gtk_tree_model_get_iter (model, &iter, path);
//printf("path is %s=%s\n", path_string, new_text );
	gtk_tree_model_get (model, &iter, 0, &pDef, -1);
//printf("b 1\n" );
//printf("ptr is %lx\n", pDef );
	g_assert(NULL!=pDef);
//printf("cmp %s %s\n", pDef->title, new_text);
	/*if (strcmp (pDef->title, new_text) != 0) {
printf("b 2\n" );	
		view_column_set_title( pDef, new_text );
printf("b 3\n" );
	}*/
	txt = g_strdup_printf( "\n%s", new_text );
	view_column_set_title( pDef, txt );
	g_free(txt);
//printf("b 4\n" );	
	gtk_tree_path_free (path);
}

static void
task_columns_dialog_get_data_showncol(GtkTreeViewColumn *tree_column,
				      GtkCellRenderer   *cell,
				      GtkTreeModel      *tree_model,
				      GtkTreeIter       *iter,
				      gpointer           data)
{
	ViewColumnDef	*pDef =NULL ;
	gchar           *text=NULL;
	gtk_tree_model_get (tree_model, iter, 0, &pDef, -1);
	g_assert(NULL!=pDef);
	switch((int)data)
	{
	case EDIT_TASK_COLUMNS_ID:
		if(NULL!=pDef)
			text=g_strdup(view_column_get_type_string_const(pDef));
		else
			text=g_strdup("null");
		break;
	case EDIT_TASK_COLUMNS_TITLE:
		if(NULL!=pDef)
		{
			text=view_column_get_title_transl_dup(pDef);
		}else
			text=g_strdup("null");
		break;
	default:
		g_warning ("Type not implemented.");
		text=g_strdup("?");
		break;
	}
	g_object_set (cell, "text", text, NULL);
	g_free(text);
}

static void
task_columns_dialog_setup_columns_list (ECDialogData *data)
{
	GtkTreeView       *tree;
	GtkTreeViewColumn *col;
	GtkCellRenderer   *cell;
	GtkTreeModel      *model;
	GtkListStore      *store;
	GtkTreeIter       iter;

	tree = GTK_TREE_VIEW (data->w_shown_columns_list);
	gtk_tree_selection_set_mode (gtk_tree_view_get_selection (tree),
				     GTK_SELECTION_SINGLE);
	GList             *properties, *l;
	store = gtk_list_store_new (1/*NUMBER_OF_EDIT_TASK_COLUMNS_COLS*/, G_TYPE_POINTER);
	model = GTK_TREE_MODEL (store);
	gtk_tree_view_set_model (tree, model);

	for (l = data->list_shown_columns; NULL != l; l = l->next) {
		ViewColumnDef* col_def = (ViewColumnDef*) l->data ;
		gtk_list_store_append (store, &iter);

		/*gtk_list_store_set (store,
				    &iter,
				    EDIT_TASK_COLUMNS_ID, view_column_get_translated_string(col_def),
				    EDIT_TASK_COLUMNS_TITLE, col_def->title,
				    EDIT_TASK_PTR, col_def,
				    -1);*/
		gtk_list_store_set (store,
				    &iter,
				    0/*EDIT_TASK_PTR*/, col_def,
				    -1);
//printf("ptr ins is %lx\n", col_def );
	}
	/* Type */
	cell = gtk_cell_renderer_text_new ();
	g_object_set (cell, "editable", FALSE, NULL);
	col = gtk_tree_view_column_new_with_attributes (_("Type"), cell, NULL);
	gtk_tree_view_column_set_cell_data_func (col, cell, 
						 (GtkTreeCellDataFunc) task_columns_dialog_get_data_showncol,
						 (gpointer)EDIT_TASK_COLUMNS_ID, 0);
	gtk_tree_view_append_column (tree, col);

	cell = gtk_cell_renderer_text_new ();
	g_object_set (cell, "editable", TRUE, NULL);
	/* Label */
	col = gtk_tree_view_column_new_with_attributes (_("Label"),cell, NULL);
	gtk_tree_view_column_set_cell_data_func (col, cell, 
						 (GtkTreeCellDataFunc) task_columns_dialog_get_data_showncol,
						 (gpointer)EDIT_TASK_COLUMNS_TITLE, 0);
	g_signal_connect (cell,
			  "edited",
			  G_CALLBACK (task_columns_dialog_label_edited),
			  data);
	gtk_tree_view_append_column (tree, col);
}

static void
columns_dialog_update_title (ECDialogData *data, const PlannerViewColumnsEnum view_type )
{
	gchar *title;

	if ( COLUMNS_TASK_VIEW == view_type ) {
		title = g_strdup (_("Edit task view columns"));
	} else {
		title = g_strdup (_("Edit gantt view columns"));
	}

	gtk_window_set_title (GTK_WINDOW (data->dialog), title);

	g_free (title);
}	

static void
task_dialog_parent_destroy_cb (GtkWidget *parent,
			       GtkWidget *dialog)
{
	gtk_widget_destroy (dialog);
}

static void
task_dialog_destroy_cb (GtkWidget  *parent,
			ECDialogData *data)
{
}

GList *
planner_edit_task_columns (          PlannerWindow           *window,
			       const PlannerViewColumnsEnum   view_type,
			             GList                   *column_list,
			             GList                   *actual_shown_columns )

{
	ECDialogData *data;
	GladeXML     *glade;
	GtkWidget    *dialog;
	GList        *resList = NULL;
	gboolean      isReturnList = FALSE;
	
	glade = glade_xml_new (GLADEDIR "/task-columns-dialog.glade",
			       NULL,
			       GETTEXT_PACKAGE);
	if (!glade) {
		g_warning ("Could not create task columns dialog.");
		return NULL;
	}

	dialog = glade_xml_get_widget (glade, "task_columns_dialog");

	data = g_new0 (ECDialogData, 1);
	/* changes are done on the duoplicate list */
	data->list_shown_columns = view_columns_duplicate(actual_shown_columns) ;
	data->list_columns= column_list;

	data->main_window = window;
	data->dialog = dialog;

	g_signal_connect (dialog,
			  "destroy",
			  G_CALLBACK (task_dialog_destroy_cb),
			  data);
	
	/*TODO g_signal_connect_object (window,
				 "destroy",
				 G_CALLBACK (task_dialog_parent_destroy_cb),
				 dialog,
				 0);*/

	data->w_shown_columns_list   = glade_xml_get_widget (glade, "columns_tree");
	data->w_hidden_columns_list  = glade_xml_get_widget (glade, "columnstoshow");
	data->w_move_up_button       = glade_xml_get_widget (glade, "moveup");
	data->w_move_down_button     = glade_xml_get_widget (glade, "movedown");
	data->w_add_button           = glade_xml_get_widget (glade, "add_columns");
	data->w_remove_button        = glade_xml_get_widget (glade, "remove");

	task_columns_dialog_setup_columns_list (data);

/*	g_signal_connect ( dialog->move_up,
			  "clicked",
			  G_CALLBACK (columns_dialog_move_up),
			  data);
	g_signal_connect ( dialog->move_down,
			  "clicked",
			  G_CALLBACK (columns_dialog_move_down),
			  data);
	g_signal_connect ( dialog->remove_button,
			  "clicked",
			  G_CALLBACK (columns_dialog_remove),
			  data);*/

	g_object_set_data_full (G_OBJECT (dialog),
				"data", data,
				g_free);

	/*TODO columns_dialog_setup_widgets (data, glade);*/

	/*TODO columns_dialog_update_sensitivity (data);*/

	columns_dialog_update_title(data, view_type );
	resList = NULL ;
	if(NULL!=dialog)
	{
		gint result = gtk_dialog_run (GTK_DIALOG (dialog));
		if(GTK_RESPONSE_OK==result)
    		{
			/* build the resulting list */
			resList = data->list_shown_columns;
			isReturnList = TRUE;
		}
		gtk_widget_destroy (dialog); /*fatto in automatico alla chiusura da close/quit ?*/
	}
	if( !isReturnList && (NULL!=data->list_shown_columns) )
	{
		view_columns_end(data->list_shown_columns);
	}
	return resList;
}
